import os
import sys
import functools
import asyncio
import json
import argparse
import modconf
import subprocess
import time
import pickle
import logging.config

import ws_storage.impl.client

class Foo(object): pass

class TestClient():

    def setUp(self):
        self.conf_mod = 'ws_storage.tests.conf.simple'

        self.p = subprocess.Popen((
            'ws_storage',
            'runserver',
            self.conf_mod, 
            '--conf_dir', 
            os.path.join(os.environ['HOME'], 'config'), '-d'))

        time.sleep(3)
        
        self.loop = asyncio.new_event_loop()
        asyncio.set_event_loop(self.loop)

    def tearDown(self):
        self.p.kill()

        print('close loop')
        self.loop.close()

    def test(self):
        try:
            self.setUp()
            self.loop.run_until_complete(self.go())
        finally:
            self.tearDown()

    async def go(self):
        conf = modconf.import_class(
                self.conf_mod, 
                'Conf', 
                ('DEVELOP', os.path.join(os.environ['HOME'], 'config')))

        #client = ws_sheets_server.sockets.Client(port)
        
        #print(client.book_new())
        
        L = conf.LOGGING
        for l in L['loggers'].values():
            l['handlers'] = ['console']
        logging.config.dictConfig(L)
        
        logger = logging.getLogger(__name__)

        logger.debug('ws_stroage version = {}'.format(ws_storage.__version__))

        protocol = await ws_storage.impl.client.Storage.create(self.loop, 'localhost', conf.PORT)

        print(protocol)
        
        if protocol.transport.is_closing():
            self.fail()

        lst = await protocol.list_files()

        assert lst is not None

        o = Foo()
        
        print('write')
        
        b = pickle.dumps(o)

        future = protocol.write_binary('0', b)
        print(future)
        res = await future
        print(res)

        print('read')
        future = protocol.read_binary('0')
        print(future)
        res = await future
 
        print(res)

        o = pickle.loads(b)

        print(o)
   
        assert isinstance(o, Foo)

    
    
    
    
